﻿namespace Cellfish.Toolbox.Facts.Test
{
    using System;
    using System.Threading.Tasks;

    using Cellfish.Test;

    using Microsoft.VisualStudio.TestTools.UnitTesting;

    [TestClass]
    public class EnsureFacts
    {
        [TestMethod, ExpectedException(typeof(AssertFailedException))]
        public void When_ensuring_exception_is_thrown_and_no_action_is_provided_Then_assertion_fails()
        {
            Ensure.Throws(null, _ => { });
        }

        [TestMethod, ExpectedException(typeof(AssertFailedException))]
        public void When_ensuring_exception_is_thrown_and_no_validation_is_provided_Then_assertion_fails()
        {
            Ensure.Throws(() => { throw new Exception("Dummy Exception"); }, null);
        }

        [TestMethod, ExpectedException(typeof(AssertFailedException))]
        public void When_ensuring_exception_is_thrown_and_no_noException_action_is_provided_Then_assertion_fails()
        {
            Ensure.Throws(() => { throw new Exception("Dummy Exception"); }, _ => { }, null);
        }

        [TestMethod, ExpectedException(typeof(AssertFailedException))]
        public void When_ensuring_exception_is_thrown_and_no_exception_is_thrown_Then_assertion_fails()
        {
            Ensure.Throws(() => { }, _ => { });
        }

        [TestMethod]
        public void When_ensuring_exception_is_thrown_and_validation_does_not_assert_Then_no_assertion_fails()
        {
            Ensure.Throws(() => { throw new Exception("Dummy Exception"); }, _ => { });
        }

        [TestMethod, ExpectedException(typeof(AssertFailedException))]
        public void When_ensuring_specific_exception_is_thrown_and_child_exception_is_thrown_Then_assertion_fails()
        {
            Ensure.Throws<ArgumentException>(() => { throw new ArgumentNullException(); });
        }

        [TestMethod, ExpectedException(typeof(AssertFailedException))]
        public void When_ensuring_task_fails_and_task_does_not_fail_Then_assertion_fails()
        {
            Ensure.TaskFails<Exception>(Task.FromResult(42));
        }

        [TestMethod, ExpectedException(typeof(AssertFailedException))]
        public void When_ensuring_task_fails_and_task_fails_with_wrong_exception_Then_assertion_fails()
        {
            var task = Task.Factory.StartNew(() => { throw new TimeoutException(); });
            Ensure.TaskFails<ArgumentException>(task);
        }

        [TestMethod]
        public void When_ensuring_task_fails_and_task_fails_with_that_exception_Then_no_assertion_fails()
        {
            var task = Task.Factory.StartNew(() => { throw new TimeoutException(); });
            Ensure.TaskFails<TimeoutException>(task);
        }
    }
}
